function [angles,error,symbol_errors] = MLdetector(sampled_values, symbols,p,start_angle,start_sum)
% Create empty tables
theta_hat_table = zeros(1,length(sampled_values));
total_angle_table = zeros(1,length(sampled_values));
theta_hat=start_angle;
errors = 0; safe_errors = 0; safe_correct = 0; v_hat = 0; freq_est = 0*p.f_T;
temp_sum = start_sum; freq_est_addon = 0; safe_sample = 0;

for k=1:length(sampled_values)
    % Correct frequency
    if p.bootstrapping == 1
        sampled_values(k) = sampled_values(k)*exp(-1i*v_hat);
    end

    if p.open_loop == 1
        y_k = sampled_values(k); % Open loop
    else
        y_k = sampled_values(k)*exp(-1i*theta_hat);
    end
    
    if p.ML_system == 1 && k <= 31
        % Training sequence
        c_hat_k = symbols(k);
        safe_sample = 1;
        
        temp_sum = temp_sum + conj(c_hat_k)*sampled_values(k);
        theta_hat = angle(temp_sum);
    else
        % Data sequence
        [c_hat_k,safe_sample] = detectorQAM(y_k,16,p);

        % Count safe sample errors and correct samples
        if p.costas_safe_module == 1 && safe_sample == 1
            if c_hat_k ~= symbols(k)
                safe_errors = safe_errors +1;
            else
                safe_correct = safe_correct +1;
            end
        end
        
        if p.costas_safe_module == 1
            % Only update estimate if the sample is close enough
            % to the constellation point
            if safe_sample == 1
                temp_sum = temp_sum + conj(c_hat_k)*sampled_values(k);
                theta_hat = angle(temp_sum);
            end
        else
            temp_sum = temp_sum + conj(c_hat_k)*sampled_values(k);
            theta_hat = angle(temp_sum);
        end
    end

    % Store estimated phase
    theta_hat_table(k) = theta_hat;
    total_angle_table(k) = theta_hat + v_hat;

    % Bootstrapping system
    if p.bootstrapping == 1
        interval = 30;
        if ~mod(k,interval)
            prev_freq_est_addon = freq_est_addon;
			% Estimate frequency
            freq_est_addon = estimatefreq(theta_hat_table(k-(interval-1):k));
            freq_correct = freq_est_addon;
            freq_est_addon = freq_est_addon + prev_freq_est_addon;            
           
            % Calculate new symbols
            new_seq = addfreq(sampled_values(k-(interval-1):k),-freq_correct);
            new_symbols = symbols(k-(interval-1):k);
            for i=1:interval
                % Only correct data-symbols
                if ~(p.ML_system == 1 && (k-interval+i)<=31)
                    [c_hat_k,safe_sample] = detectorQAM(new_seq(i),16,p);
                    % Count transmission errors
                    if c_hat_k ~= new_symbols(i)
                        errors = errors +1;
                    end
                end
            end
        end
    else
        % Count transmission errors
        if c_hat_k ~= symbols(k)
           errors = errors +1;
        end
    end
    % Add freq. est. to each sample
    v_hat = v_hat + freq_est_addon;
end
symbol_errors = errors;
error.safe_errors = safe_errors;
error.safe_correct = safe_correct;
% Return correct table
if p.bootstrapping == 1
    angles = total_angle_table;
else
    angles = theta_hat_table;
end
