function [angles,error,symbol_errors] = costasloop(sampled_values, symbols,p,start_angle)
% Fetches parameters
gamma = p.costas_gamma;
rho = p.costas_rho;

% Create empty-arrays
e = zeros(1,length(sampled_values));
theta_hat_table = zeros(1,length(sampled_values));
theta_hat=start_angle;
symbol_errors = 0; safe_errors = 0; safe_correct = 0; dloop = 0;
for k=1:length(sampled_values)
	% Store values
    theta_hat_table(k) = theta_hat;
    if p.open_loop == 1
        y_k = sampled_values(k); % Open loop
    else
        y_k = sampled_values(k)*exp(-1i*theta_hat);
    end
    if p.KDA_system == 1 && k <= 31
        c_hat_k = symbols(k);
        safe_sample = 1;
    elseif p.Costas_Loop_training == 1
        c_hat_k = symbols(k);
        safe_sample = 1;
    else
        [c_hat_k,safe_sample] = detectorQAM(y_k,16,p);
    end
    % Count safe sample errors and approved samples
    if p.costas_safe_module == 1 && safe_sample == 1
        if c_hat_k ~= symbols(k)
            safe_errors = safe_errors +1;
        else
            safe_correct = safe_correct +1;
        end
    end
    % Count transmission errros
    if c_hat_k ~= symbols(k)
       symbol_errors = symbol_errors +1;
    end
    
    if p.costas_safe_module == 1
        if safe_sample == 1
	        % Calculate error signal
	        % Equation (5.3.19) [3]
	        e(k) = imag(conj(c_hat_k)*y_k);
	        % SECOND ORDER COSTAS LOOP
	        dloop = dloop + e(k)*gamma*rho;
	        xi = e(k)*gamma + dloop; 
	        % Equation (5.3.73) [3]
	        theta_hat = theta_hat + xi;
        end
    else
        % Calculate error signal
        % Equation (5.3.19) [3]
        e(k) = imag(conj(c_hat_k)*y_k);
        % SECOND ORDER COSTAS LOOP
        dloop = dloop + e(k)*gamma*rho;
        xi = e(k)*gamma + dloop; 
        % Equation (5.3.73) [3]
        theta_hat = theta_hat + xi;
    end
end

% Return values
error.e = e;
error.safe_errors = safe_errors;
error.safe_correct = safe_correct;
angles = theta_hat_table;

