% Simulator: Plots phase tracking progess for severeal SNRs
clear all
disp(' ')
disp('Starting simulation for phase-recovery')
% Set necessary parameters and put in struct
number_of_periods = 1000;            % Number of symbols/periods (31)
params.nr_of_periods = number_of_periods;
params.u_n = 0;                     % Mean value of noise
params.a = 0.345;                   % Roll-off factor for filters
params.phase_error = 1*4*pi/320;    % Add phase error
params.phase_stdev = 0*1/100;		% Variance in phase
params.f_T = 0*2*pi*0.00004;   		% Frequency error pr. sample
params.added_error = params.phase_error + params.phase_stdev*randn(1,number_of_periods) + params.f_T*(1:number_of_periods);
% Choose estimation technique
params.ML_estimation = 0;			% ML: Only known 2-PSK samples
params.ML_system = 0;				% ML: 31 2-PSK known before 16-QAM data
params.ML_estimation_DD = 0;		% ML: Only 16-QAM data
    params.bootstrapping = 0;       % ML: Bootstrapping module
params.Costas_Loop_training = 0;	% CL: Only known 2-PSK samples
params.Costas_Loop = 1;             % CL: Only 16-QAM data, 2nd order filter
params.KDA_system = 0;				% CL: 31 2-PSK known before 16-QAM data

params.open_loop = 0;				% Disconnect feedback path if 1

nr_of_simulations = 1;      		% Number of different simulations
nr_of_runs = 100;          			% Number of iteration per simulation

% SNR_table = [20 15 10];
SNR_table = 10*ones(1,nr_of_simulations); 		% SNR for each simulation
gamma_table = 2^(-8)*ones(1,nr_of_simulations);	% Gamma for each simulation
% gamma_table = [2^-6 2^-6 2^-7 2^-8 2^-8];
% rho_table =   [2^-7 2^-8 2^-8 2^-6 2^-8];
rho_table = 2^(-6)*ones(1,nr_of_simulations);	% Rho for each simulation
safe_distance_table = 0.0*ones(1,nr_of_simulations); % Sets safe distance boxes R = 0 tunrs system off

% No parameters to set by the user below this point
sigma2_n_table = 10.^(-SNR_table/10);
sim1_SNR = SNR_table;

% Resetting tables
sim1_tracking = zeros(nr_of_simulations,number_of_periods);
sim1_variance = zeros(nr_of_simulations,number_of_periods);
sim1_symbol_errors = zeros(nr_of_simulations,nr_of_runs);
sim1_safe_samples_e = zeros(nr_of_simulations,nr_of_runs);
sim1_safe_samples_c = zeros(nr_of_simulations,nr_of_runs);
h = waitbar(0,'Starting...');
for k=1:nr_of_simulations
    results = zeros(nr_of_runs, number_of_periods);
    symbol_errors = zeros(1,nr_of_runs);

    params.costas_gamma = gamma_table(k);
    params.costas_rho = rho_table(k);
    params.sigma_n = sigma2_n_table(k);
    params.safe_distance = safe_distance_table(k)/sqrt(5);
    if params.safe_distance == 0
       params.costas_safe_module = 0;
    else
       params.costas_safe_module = 1;
    end
    for i=1:nr_of_runs
        waitbar(((k-1)*nr_of_runs+i)/(nr_of_simulations*nr_of_runs),h,['Running simulation ' num2str(k) ' of ' num2str(nr_of_simulations)])
        % Run the system
        [results(i,:),symbol_errors(i),errors_struct] = trsystem(params);
        if params.ML_estimation == 0
            sim1_safe_samples_e(k,i) = errors_struct.safe_errors;
            sim1_safe_samples_c(k,i) = errors_struct.safe_correct;
        end
    end
    if nr_of_runs == 1
        % Because of the mean() operation on single dimension arrays
        plot_results = results;
    else
        plot_results = mean(results);
    end
    temp_variance = zeros(1,number_of_periods);
    for l=1:number_of_periods
        % Calculates variance of every step of the iterations
        temp_variance(l) = var(results(:,l));
    end
    sim1_tracking(k,:) = plot_results;
    sim1_variance(k,:) = temp_variance;
    sim1_symbol_errors(k,:) = symbol_errors;
end
waitbar(1,h,'DONE')
close(h)
filename = 'simulator_data.mat';
save(filename, 'sim1_tracking','sim1_SNR','sim1_variance','gamma_table','rho_table','params','nr_of_simulations','safe_distance_table')

if params.KDA_system == 1 || params.ML_system == 1
    number_of_periods =  number_of_periods -31;
end
for i=1:nr_of_simulations
    disp(['SNR = ' num2str(SNR_table(i)) 'dB => ' 'P(error) = ' num2str(sum(sim1_symbol_errors(i,:))/nr_of_runs/number_of_periods) ])
     if params.ML_estimation == 0
        if params.costas_safe_module == 1
        disp(['              P(error|safe) = ' num2str(sum(sim1_safe_samples_e(i,:))/(sum(sim1_safe_samples_c(i,:))+sum(sim1_safe_samples_e(i,:))))])
        disp(['              P(safe) = ' num2str((sum(sim1_safe_samples_e(i,:)) + sum(sim1_safe_samples_c(i,:)))/(number_of_periods*nr_of_runs)) ])
        end
        if params.ML_estimation_DD == 1 || params.ML_system == 1
            if params.bootstrapping == 1
                disp(['              ML estimation w/ bootstrapping:'])
            else
                disp(['              ML estimation:'])
            end
        else
            disp(['              Costas loop:'])
            disp(['              y = ' num2str(gamma_table(i)) ', p = ' num2str(rho_table(i))])
        end
        disp(['              R = ' num2str(safe_distance_table(i)) ])
     end
     disp(['              st.dev = ' num2str(sqrt(sim1_variance(i,end))*180/pi) ' at last symbol'])
end
make_plots