 function [angle_estimate,symbol_errors,errors_struct] = trsystem(p)
number_of_periods = p.nr_of_periods;     %number of symbols/periods
symbol_errors = 0; errors_struct = 0;

% Create symbols
if p.KDA_system == 1 || p.ML_system == 1
    training_symbols = datagenerator(31,2,'PSK');
    data_symbols = datagenerator(number_of_periods-31,16,'QAM');
    symbols = [training_symbols data_symbols];
elseif p.Costas_Loop_training == 1
    symbols = datagenerator(number_of_periods,2,'PSK');
else
    % Gets random symbols from constellations (complex values)
    symbols = datagenerator(number_of_periods,16,'QAM');
end

% Get filter response for pulse shaping filter
T = 1/1;
t_res = T/10;               % resolution in time for filter
g_width = 10;				  % width of pulse
g_tx = pulseshape(T,p.a,t_res,g_width);
g_rx = g_tx;                  % because of symmetric response the matched filter is equal

% Creates symbols as dirac pulses
c = zeros(1,number_of_periods*(T/t_res));
c(T/t_res+1:T/t_res:T/t_res*number_of_periods+1) = symbols;
% Puts the symbols through the pulse shaping filter (complex values)
s = conv(c,g_tx);

% Sends signal through channel, noise added
[r,just_noise] = channel(s, p.u_n, p.sigma_n);
% Add phase error
len = number_of_periods*T/t_res + 1 + 2*T/t_res*g_width;
added_error = p.phase_error + p.phase_stdev*randn(1,len) + p.f_T*number_of_periods/len*(1:len);
r = r.*exp(1i*added_error);
	
% Filters received signal in matched filter
x = conv(r,g_rx);

% Sample the signal
x_I = real(x);
x_Q = imag(x);

first_sample = 1+ 2*g_width*T/t_res + T/t_res;
sample_dist = T/t_res;
last_sample = 1+ 2*g_width*T/t_res + T/t_res*number_of_periods;
sample_time = first_sample:sample_dist:last_sample;
I_samples = x_I(sample_time);
Q_samples = x_Q(sample_time);

if p.ML_estimation == 1
    % PHASE ESTIMATOR
    % Creating c*(k)
    c_conj = conj(symbols);
    % Product c*(k)x(k)
    temp = c_conj.*(I_samples + 1i*Q_samples);
    % Summing the vector
    temp_sum = sum(temp);
    % Finding the angle
    theta_hat = angle(temp_sum);
    % Create table which shows the phase tracking
    angle_t = calcangles(temp);
elseif p.ML_system == 1 || p.ML_estimation_DD == 1
    [angle_t,errors, symbol_errors] = MLdetector(I_samples +1i*Q_samples,symbols,p,0,0);
elseif p.Costas_Loop_training == 1 ||  p.Costas_Loop == 1 || p.KDA_system == 1
    [angle_t,errors, symbol_errors] = costasloop(I_samples +1i*Q_samples,symbols,p,0);
end

% Return values
angle_estimate = angle_t;
errors_struct = errors;